﻿//// MxWDB
//// 1st Gen MxW Database Generator
//// BY MIKX
//// https://mxgit.ovh/mikx/MxWDB
//// https://dev.mxwow.ovh
//// License: GNU General Public License 3.0

using MxWDB.JSON;
using MxWDB.Utilities;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Diagnostics.Eventing.Reader;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.IO;

namespace MxWDB
{
    internal class Program
    {
        public static string version = "1.0.0_WoW11.0";

        public static int line = 0;

        public static bool serverloop = true;
        public static string state = "init";

        public static string oauth_token = "";
        static void Main(string[] args)
        {           
            //// Main Server Loop
            while (serverloop)
            {
                switch (state)
                {                   
                    case "init":
                        MSG.Splash();
                        if (IO.FirstRun())
                        {                           
                            IO.CreateDirectory("json");
                            IO.CreateDirectory("logs");
                            MSG.CMW($"[{state}] Initializing the ecosystem...", true, 1);
                            MSG.CMW($"[{state}] Open \"json/settings.json\" and edit it with your own informations.", true, 1);
                            MSG.CMW($"[{state}] Press any key to exit...", true, 1);
                            serverloop = false;
                            Console.ReadKey();
                        } else
                        {
                            if (IO.SettingsValid())
                            {
                                MSG.CMW($"[{state}] Your Settings appears to be valid.", true, 2);
                                MSG.CMW($"[{state}] DBHost:  {JSON.Settings.GetDBHost()}", true, 2);
                                MSG.CMW($"[{state}] DBPort:  {JSON.Settings.GetDBPort()}", true, 2);
                                MSG.CMW($"[{state}] DBName:  {JSON.Settings.GetDBName()}", true, 2);
                                MSG.CMW($"[{state}] DBUser:  {JSON.Settings.GetDBUser()}", true, 2);
                                MSG.CMW($"[{state}] Region:  {JSON.Settings.GetRegion()}", true, 2);
                                MSG.CMW($"[{state}] RealmID: {JSON.Settings.GetRID()}", true, 2);
                                state = "oauth";
                            } else
                            {
                                MSG.CMW($"[{state}] Your Settings appears to be invalid.", true, 3);
                                MSG.CMW($"[{state}] Open \"json/settings.json\" and edit it with your own informations.", true, 3);
                                MSG.CMW($"[{state}] Press any key to exit...", true, 3);
                                serverloop = false;
                                Console.ReadKey();
                            }
                        }                        
                        break;                   
                    case "oauth":
                        MSG.CMW($"[{state}] Getting an access token from BNET...", true, 1);
                        oauth_token = OAuth.GetAccessToken(JSON.Settings.GetAPIKey(), JSON.Settings.GetAPISecret());
                        MSG.CMW($"[{state}] Access Token: {oauth_token}.", true, 2);
                        if (args.Contains("--token")) { state = "token"; } else { state = "dump"; }                       
                        break;
                    /// ARGS - WoW Token
                    case "token":
                        MSG.CMW($"[{state}] Getting the latest Token info from BNET...", true, 1);
                        Token.GetToken(oauth_token);
                        int v = JsonConvert.DeserializeObject<Token.Root>(File.ReadAllText("json/token.json")).price;
                        DB.UpdateTokenValue(v);
                        int copper = v % 100;
                        v = (v - copper) / 100;
                        int silver = v % 100;
                        int gold = (v - silver) / 100;
                        MSG.CMW($"[{state}] WoW Token from region {Settings.GetRegion()} is worth {gold}g.", true, 1);                       
                        serverloop =false;
                        break;
                    case "dump":
                        if (!JSON.Settings.GetDebug())
                        {
                            MSG.CMW($"[{state}] Downloading the latest AH dump from BNet...", true, 1);
                            JSON.AHUrl.GetDump(JSON.Settings.GetAPIKey(), JSON.Settings.GetAPISecret(), oauth_token);
                            MSG.CMW($"[{state}] Downloading the latest AH Commodities dump from BNet...", true, 1);
                            JSON.AHUrl.GetDumpCom(JSON.Settings.GetAPIKey(), JSON.Settings.GetAPISecret(), oauth_token);
                            if (IO.FileIsDownloaded("json/ah.dump.json") && IO.FileIsDownloaded("json/ah.com.dump.json"))
                            {
                                MSG.CMW($"[{state}] Both Dump appears to be correctly downloaded.", true, 2);
                                state = "checkdb";
                            }
                            else
                            {
                                MSG.CMW($"[{state}] One or both of the dump failed to download. Retrying in 10 seconds...", true, 3);
                                Thread.Sleep(10000);
                                state = "dump";
                            }
                        } else
                        {
                            MSG.CMW($"[{state}] Debug Enabled. Skipping Dump.", true, 2);
                            state = "checkdb";
                        }                     
                        break;
                    case "checkdb":
                        MSG.CMW($"[{state}] Checking if we can connect to the DB...", true, 1);
                        if (DB.DatabaseConnected())
                        {
                            MSG.CMW($"[{state}] We can connect to the DB.", true, 2);
                            MSG.CMW($"[{state}] Cleaning the DB...", true, 1);
                            DB.CleanDB();
                            state = "genintable";
                        } else
                        {
                            MSG.CMW($"[{state}] We can't connect to the DB.", true, 3);
                            MSG.CMW($"[{state}] Open \"json/settings.json\" and edit the DB informations.", true, 3);
                            MSG.CMW($"[{state}] Press any key to exit...", true, 3);
                            serverloop = false;
                            Console.ReadKey();
                        }
                        break;
                    case "genintable":
                        MSG.CMW($"[{state}] Generating the internal table from Auctions Data...", true, 1);
                        JSON.AHDump.GenTable();
                        MSG.CMW($"[{state}] Generating the internal table from Commodities Data...", true, 1);
                        JSON.AHDump.GenTableCom();
                        state = "gendbtable";
                        break;
                    case "gendbtable":
                        MSG.CMW($"[{state}] Generating the DB tables from Auctions Data...", true, 1);
                        DB.GenItemsTable();
                        DB.GenListingTable();
                        MSG.CMW($"[{state}] Generating the DB tables from Commodities Data...", true, 1);
                        DB.GenItemsTableCom();
                        DB.GenListingTableCom();
                        state = "genweb";
                        break;
                    case "genweb":
                        MSG.CMW($"[{state}] Generating the Index Table...", true, 1);
                        JSON.IndexGen.GenIndex("json/index.json");
                        MSG.CMW($"[{state}] Generating the Special Table...", true, 1);
                        JSON.SpecialGen.GenSpecial("json/special.json");
                        serverloop=false;
                        break;
                }
            }
        }
    }
}
