﻿using MxWDB.JSON;
using MySqlConnector;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using System.Net;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;
using System.Web.Management;
using System.Xml.Serialization;
using static MxWDB.JSON.AHDump;

namespace MxWDB.Utilities
{
    [XmlRoot(ElementName = "quality")]
    public class XMLQuality
    {

        [XmlAttribute(AttributeName = "id")]
        public int Id { get; set; }

        [XmlText]
        public string Text { get; set; }
    }
    public class Appearance
    {
        public Key key { get; set; }
        public int id { get; set; }
    }

    public class AttackSpeed
    {
        public int value { get; set; }
        public string display_string { get; set; }
    }

    public class Binding
    {
        public string type { get; set; }
        public string name { get; set; }
    }

    public class Color
    {
        public int r { get; set; }
        public int g { get; set; }
        public int b { get; set; }
        public double a { get; set; }
    }

    public class Damage
    {
        public int min_value { get; set; }
        public int max_value { get; set; }
        public string display_string { get; set; }
        public DamageClass damage_class { get; set; }
    }

    public class DamageClass
    {
        public string type { get; set; }
        public string name { get; set; }
    }

    public class Display
    {
        public string display_string { get; set; }
        public Color color { get; set; }
    }

    public class DisplayStrings
    {
        public string header { get; set; }
        public string gold { get; set; }
        public string silver { get; set; }
        public string copper { get; set; }
    }

    public class Dps
    {
        public double value { get; set; }
        public string display_string { get; set; }
    }

    public class Durability
    {
        public int value { get; set; }
        public string display_string { get; set; }
    }

    public class InventoryType
    {
        public string type { get; set; }
        public string name { get; set; }
    }

    public class Item
    {
        public Key key { get; set; }
        public int id { get; set; }
    }

    public class ItemClass
    {
        public Key key { get; set; }
        public string name { get; set; }
        public int id { get; set; }
    }

    public class ItemSubclass
    {
        public Key key { get; set; }
        public string name { get; set; }
        public int id { get; set; }
    }

    public class Key
    {
        public string href { get; set; }
    }

    public class Level
    {
        public int value { get; set; }
        public string display_string { get; set; }
    }

    public class Links
    {
        public Self self { get; set; }
    }

    public class Media
    {
        public Key key { get; set; }
        public int id { get; set; }
    }

    public class PreviewItem
    {
        public Item item { get; set; }
        public int context { get; set; }
        public List<int> bonus_list { get; set; }
        public Quality quality { get; set; }
        public string name { get; set; }
        public Media media { get; set; }
        public ItemClass item_class { get; set; }
        public ItemSubclass item_subclass { get; set; }
        public InventoryType inventory_type { get; set; }
        public Binding binding { get; set; }
        public Weapon weapon { get; set; }
        public List<Stat> stats { get; set; }
        public SellPrice sell_price { get; set; }
        public Requirements requirements { get; set; }
        public Level level { get; set; }
        public Durability durability { get; set; }
    }

    public class Quality
    {
        public string type { get; set; }
        public string name { get; set; }
    }

    public class Requirements
    {
        public Level level { get; set; }
    }

    public class Root
    {
        public Links _links { get; set; }
        public int id { get; set; }
        public string name { get; set; }
        public Quality quality { get; set; }
        public int level { get; set; }
        public int required_level { get; set; }
        public Media media { get; set; }
        public ItemClass item_class { get; set; }
        public ItemSubclass item_subclass { get; set; }
        public InventoryType inventory_type { get; set; }
        public int purchase_price { get; set; }
        public int sell_price { get; set; }
        public int max_count { get; set; }
        public bool is_equippable { get; set; }
        public bool is_stackable { get; set; }
        public PreviewItem preview_item { get; set; }
        public int purchase_quantity { get; set; }
        public List<Appearance> appearances { get; set; }
    }

    public class Self
    {
        public string href { get; set; }
    }

    public class SellPrice
    {
        public int value { get; set; }
        public DisplayStrings display_strings { get; set; }
    }

    public class Stat
    {
        public Type type { get; set; }
        public int value { get; set; }
        public bool is_negated { get; set; }
        public Display display { get; set; }
    }

    public class Type
    {
        public string type { get; set; }
        public string name { get; set; }
    }

    public class Weapon
    {
        public Damage damage { get; set; }
        public AttackSpeed attack_speed { get; set; }
        public Dps dps { get; set; }
    }
    internal class DB
    {
        public static MySqlConnection ItemDB()
        {
            MySqlConnection conn;
            string myConnectionString;

            myConnectionString = string.Format("server={0};port={1};uid={2};pwd={3};database={4};", JSON.Settings.GetDBHost(), JSON.Settings.GetDBPort(), JSON.Settings.GetDBUser(), JSON.Settings.GetDBPass(), JSON.Settings.GetDBName());
            conn = new MySqlConnection();
            conn.ConnectionString = myConnectionString;
            return conn;
        }

        public static bool DatabaseConnected()
        {
            MySqlConnection mcc = ItemDB();
            try
            {
                mcc.Open();
                var temp = mcc.State.ToString();
                MSG.CMW(temp, true,4);
                if (mcc.State == ConnectionState.Open && temp == "Open")
                {
                    mcc.Close();
                    return true;
                }
                else
                {
                    return false;
                }
            }
            catch (MySqlException ex)
            {
                MSG.CMW($"[db] {ex.Message}", true, 3);
                return false;
            }
        }

        public static void GenItemsTable()
        {
            string stm = "SELECT * FROM items";
            MySqlConnection mc = ItemDB();
            MySqlCommand cmd = new MySqlCommand(stm, mc);
            mc.Open();

            DataTable data = new DataTable();
            data.Load(cmd.ExecuteReader());

            using (MySqlConnection mcc = ItemDB())
            {
                StringBuilder sCommand = new StringBuilder("INSERT INTO ah_items (itemid,maximum,median,minimum,quantity) VALUES ");
                List<string> Rows = new List<string>();
                DataRow[] result = JSON.AHDump.aucds.Tables["auclist"].Select("iid is not null");
                foreach (DataRow row in data.Rows)
                {
                    Int64 median = 0;
                    Int64 maximum = 0;
                    Int64 minimum = 0;
                    int i = Convert.ToInt32(row["id"]);
                    int ie = JSON.AHDump.aucds.Tables[0].Select(string.Format("iid = '{0}'", i)).Length;
                    if (ie >= 1)
                    {
                        if (JSON.AHDump.aucds.Tables["auclist"].Compute("AVG(uv)", string.Format("iid = '{0}'", i)) != DBNull.Value)
                        {
                            median = Convert.ToInt64(JSON.AHDump.aucds.Tables["auclist"].Compute("AVG(uv)", string.Format("iid = '{0}'", i)));
                        }
                        if (JSON.AHDump.aucds.Tables["auclist"].Compute("MAX(uv)", string.Format("iid = '{0}'", i)) != DBNull.Value)
                        {
                            maximum = Convert.ToInt64(JSON.AHDump.aucds.Tables["auclist"].Compute("MAX(uv)", string.Format("iid = '{0}'", i)));
                        }
                        if (JSON.AHDump.aucds.Tables["auclist"].Compute("MIN(uv)", string.Format("iid = '{0}'", i)) != DBNull.Value)
                        {
                            minimum = Convert.ToInt64(JSON.AHDump.aucds.Tables["auclist"].Compute("MIN(uv)", string.Format("iid = '{0}'", i)));
                        }
                        int qty = JSON.AHDump.aucds.Tables["auclist"].Select(string.Format("iid = '{0}'", i)).Length;
                        Rows.Add(string.Format("('{0}','{1}','{2}','{3}','{4}')", i, maximum, median, minimum, qty));
                    }
                }
                sCommand.Append(string.Join(",", Rows));
                sCommand.Append(";");
                mcc.Open();
                using (MySqlCommand myCmd = new MySqlCommand(sCommand.ToString(), mcc))
                {
                    myCmd.CommandType = CommandType.Text;
                    myCmd.ExecuteNonQuery();
                }
            }
        }

        public static void GenItemsTableCom()
        {
            using (MySqlConnection mcc = ItemDB())
            {
                StringBuilder sCommand = new StringBuilder("INSERT INTO ah_items (itemid,maximum,median,minimum,quantity) VALUES ");
                List<string> Rows = new List<string>();
                DataRow[] result = JSON.AHDump.aucdscom.Tables["auclistcom"].Select("iid is not null");
                for (int il = 1; il < 300000; il++)
                {
                    Int64 median = 0;
                    Int64 maximum = 0;
                    Int64 minimum = 0;
                    int ie = JSON.AHDump.aucdscom.Tables[0].Select(string.Format("iid = '{0}'", il)).Length;
                    if (ie >= 1)
                    {
                        if (JSON.AHDump.aucdscom.Tables["auclistcom"].Compute("AVG(uv)", string.Format("iid = '{0}'", il)) != DBNull.Value)
                        {
                            median = Convert.ToInt64(JSON.AHDump.aucdscom.Tables["auclistcom"].Compute("AVG(uv)", string.Format("iid = '{0}'", il)));
                        }
                        if (JSON.AHDump.aucdscom.Tables["auclistcom"].Compute("MAX(uv)", string.Format("iid = '{0}'", il)) != DBNull.Value)
                        {
                            maximum = Convert.ToInt64(JSON.AHDump.aucdscom.Tables["auclistcom"].Compute("MAX(uv)", string.Format("iid = '{0}'", il)));
                        }
                        if (JSON.AHDump.aucdscom.Tables["auclistcom"].Compute("MIN(uv)", string.Format("iid = '{0}'", il)) != DBNull.Value)
                        {
                            minimum = Convert.ToInt64(JSON.AHDump.aucdscom.Tables["auclistcom"].Compute("MIN(uv)", string.Format("iid = '{0}'", il)));
                        }
                        int qty = JSON.AHDump.aucdscom.Tables["auclistcom"].Select(string.Format("iid = '{0}'", il)).Length;
                        Rows.Add(string.Format("('{0}','{1}','{2}','{3}','{4}')", il, maximum, median, minimum, qty));
                    }
                }
                sCommand.Append(string.Join(",", Rows));
                sCommand.Append(";");
                mcc.Open();
                using (MySqlCommand myCmd = new MySqlCommand(sCommand.ToString(), mcc))
                {
                    myCmd.CommandType = CommandType.Text;
                    myCmd.ExecuteNonQuery();
                }
            }
        }

        public static void GenListingTable()
        {
            using (MySqlConnection mcc = ItemDB())
            {
                StringBuilder sCommand = new StringBuilder("INSERT INTO ah_listing (itemid,quantity,uvalue,tvalue) VALUES ");
                List<string> Rows = new List<string>();
                DataRow[] result = JSON.AHDump.aucds.Tables["auclist"].Select("iid is not null");
                foreach (DataRow alr in result)
                {
                    Int64 uvalue = 0;
                    Int64 tvalue = 0;
                    int i = Convert.ToInt32(alr["iid"]);
                    int quantity = Convert.ToInt32(alr["qty"]);
                    if (alr["uv"] != DBNull.Value)
                    {
                        uvalue = Convert.ToInt64(alr["uv"]);
                        tvalue = Convert.ToInt64(alr["uv"]) * Convert.ToInt64(alr["qty"]);
                    }
                    Rows.Add(string.Format("('{0}','{1}','{2}','{3}')", i, quantity, uvalue, tvalue));
                }
                sCommand.Append(string.Join(",", Rows));
                sCommand.Append(";");
                mcc.Open();
                using (MySqlCommand myCmd = new MySqlCommand(sCommand.ToString(), mcc))
                {
                    myCmd.CommandType = CommandType.Text;
                    myCmd.ExecuteNonQuery();
                }
            }
        }

        public static void GenListingTableCom()
        {
            using (MySqlConnection mcc = ItemDB())
            {
                StringBuilder sCommand = new StringBuilder("INSERT INTO ah_listing (itemid,quantity,uvalue,tvalue) VALUES ");
                List<string> Rows = new List<string>();
                DataRow[] result = JSON.AHDump.aucdscom.Tables["auclistcom"].Select("iid is not null");
                foreach (DataRow alr in result)
                {
                    Int64 uvalue = 0;
                    Int64 tvalue = 0;
                    int i = Convert.ToInt32(alr["iid"]);
                    int quantity = Convert.ToInt32(alr["qty"]);
                    if (alr["uv"] != DBNull.Value)
                    {
                        uvalue = Convert.ToInt64(alr["uv"]);
                        tvalue = Convert.ToInt64(alr["uv"]) * Convert.ToInt64(alr["qty"]);
                    }
                    Rows.Add(string.Format("('{0}','{1}','{2}','{3}')", i, quantity, uvalue, tvalue));
                }
                sCommand.Append(string.Join(",", Rows));
                sCommand.Append(";");
                mcc.Open();
                using (MySqlCommand myCmd = new MySqlCommand(sCommand.ToString(), mcc))
                {
                    myCmd.CommandType = CommandType.Text;
                    myCmd.ExecuteNonQuery();
                }
            }
        }

        public static bool CheckItemInfoExists(int id)
        {
            MySqlConnection con = ItemDB();
            string cmdText = $"SELECT * FROM items WHERE id = '{id}'";
            MySqlCommand cmd = new MySqlCommand(cmdText, con);
            con.Open();
            var row = cmd.ExecuteReader();
            if (row.HasRows) { con.Close(); return true; } else { con.Close(); return false; }
        }

        public static void GenItemInfo(int id)
        {
            int itemrarity = 0;
            if (!File.Exists($"json/items/{id}.json"))
            {               
                using (WebClient wc = new WebClient())
                {
                    ServicePointManager.ServerCertificateValidationCallback += (p1, p2, p3, p4) => true;
                    ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls12;
                    Web.DownloadFileNoAsync($"https://www.wowhead.com/classic/item={id}&xml", $"xml/items/{id}.xml");                  
                }
                XmlSerializer serializer = new XmlSerializer(typeof(XMLQuality));
                using (StreamReader reader = new StreamReader($"xml/items/{id}.xml"))
                {
                    itemrarity = Convert.ToInt32((XMLQuality)serializer.Deserialize(reader));
                }
                if (itemrarity > 0)
                {
                    MSG.CMW($"[db] Item \"{id}\" doesn't exists in our Item Info table. We will add it.", true, 4);
                    using (WebClient wc = new WebClient())
                    {
                        ServicePointManager.ServerCertificateValidationCallback += (p1, p2, p3, p4) => true;
                        ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls12;
                        Web.DownloadFileNoAsync($"https://us.api.blizzard.com/data/wow/item/{id}?namespace=static-us&locale=fr_FR&access_token={Program.oauth_token}", $"json/items/{id}.json");                  
                    }
                }              
            } else
            {
                Root ij = JsonConvert.DeserializeObject<Root>($"json/items/{id}.json");
                string name = ij.name;
                MSG.CMW($"[db] {name}", true, 4);
            }          
        }

        public static void CleanDB()
        {
            string stm2 = string.Format("DELETE FROM ah_items");
            MySqlConnection mc2 = ItemDB();
            MySqlCommand cmd2 = new MySqlCommand(stm2, mc2);
            mc2.Open();
            cmd2.ExecuteNonQuery();
            mc2.Close();

            string stm3 = string.Format("DELETE FROM ah_listing");
            MySqlConnection mc3 = ItemDB();
            MySqlCommand cmd3 = new MySqlCommand(stm3, mc3);
            mc3.Open();
            cmd3.ExecuteNonQuery();
            mc3.Close();

            string stm5 = string.Format("DELETE FROM ah_index");
            MySqlConnection mc5 = ItemDB();
            MySqlCommand cmd5 = new MySqlCommand(stm5, mc5);
            mc5.Open();
            cmd5.ExecuteNonQuery();
            mc5.Close();

            string stm6 = string.Format("DELETE FROM ah_special_data");
            MySqlConnection mc6 = ItemDB();
            MySqlCommand cmd6 = new MySqlCommand(stm6, mc6);
            mc6.Open();
            cmd6.ExecuteNonQuery();
            mc6.Close();
        }

        public static void UpdateTokenValue(int v)
        {
            string stm2 = string.Format($"DELETE FROM mxw_token WHERE region='{Settings.GetRegion()}'");
            MySqlConnection mc2 = ItemDB();
            MySqlCommand cmd2 = new MySqlCommand(stm2, mc2);
            mc2.Open();
            cmd2.ExecuteNonQuery();
            mc2.Close();

            using (MySqlConnection mcc = ItemDB())
            {
                StringBuilder sCommand = new StringBuilder($"INSERT INTO mxw_token (region,value) VALUES ('{Settings.GetRegion()}','{v}')");
                mcc.Open();
                using (MySqlCommand myCmd = new MySqlCommand(sCommand.ToString(), mcc))
                {
                    myCmd.CommandType = CommandType.Text;
                    myCmd.ExecuteNonQuery();
                }
            }
        }
    }
}
