﻿using Google.Protobuf.WellKnownTypes;
using Newtonsoft.Json;
using Org.BouncyCastle.Asn1.Cmp;
using Org.BouncyCastle.Asn1.Crmf;
using Org.BouncyCastle.Asn1.Ocsp;
using RestSharp;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.IO;

namespace MxWDB.Utilities
{
    class OAuth
    {
        public class OAuthRoot
        {
            public string access_token { get; set; }
        }

        /// <summary>
        /// Get an access token from BNET using a registered application Client ID and Secret ID.
        /// </summary>
        /// <param name="key">Your registered application Client ID.</param>
        /// <param name="secret">Your registered application Secret ID.</param>
        /// <returns></returns>
        public static string GetAccessToken(string key, string secret)
        {
            ServicePointManager.SecurityProtocol |= SecurityProtocolType.Ssl3 | SecurityProtocolType.Tls12 | SecurityProtocolType.Tls11 | SecurityProtocolType.Tls;

            RestClient client = new RestClient($"https://{JSON.Settings.GetRegion()}.battle.net/oauth/token");
            var request = new RestRequest
            {
                Method = RestSharp.Method.Post
            };
            var by = Encoding.UTF8.GetBytes($"{key}:{secret}");
            var b64 = Convert.ToBase64String(by);
            request.AddParameter("grant_type", "client_credentials");
            request.AddHeader("Authorization", $"Basic {b64}");
            request.AddHeader("Content-type", "application/x-www-form-urlencoded");
            RestResponse response = client.Execute(request);

            var tokenResponse = JsonConvert.DeserializeObject<OAuthRoot>(response.Content);

            return tokenResponse.access_token;
        }
    }
}
